#include "Renderer.h"
#include "../Scene/Components.h"
#include "../Graphics/Memory/VertexArray.h"
#include "../Graphics/Memory/Buffer.h"
#include <glad/glad.h>
#include <glm/gtc/type_precision.hpp>

namespace YoggieEngine {
float Angle = 0.0;
Camera cam = Camera(glm::vec3(12.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.0f), 90.0f);
glm::mat4 projection = glm::perspective(glm::radians(cam.Zoom), (800.0f / 600.0f), 0.001f, 100.0f);

Renderer::Renderer(){}

Renderer::~Renderer(){}

void Renderer::Prepare(Scene& scene ) {
	auto group = scene.getReg().view<Render3DComponent>();
	group.each([](auto enity, Render3DComponent& renderComponent) {
		VertexArray va = VertexArray();
		Buffer vertexBuffer = Buffer();
		Buffer elementBuffer = Buffer();

		va.Create();
		va.Bind();

		vertexBuffer.createBuffer();
		vertexBuffer.Bind(false);
		vertexBuffer.setBufferData((void*)&renderComponent.mesh.vertices[0], renderComponent.mesh.vertices.size() * sizeof(Vertex), false);

		elementBuffer.createBuffer();
		elementBuffer.Bind(true);
		elementBuffer.setBufferData((void*)&renderComponent.mesh.elements[0], renderComponent.mesh.elements.size() * sizeof(unsigned int), true);

		va.AttachAttribute(0, 3, sizeof(Vertex));

		glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, sizeof(Vertex), (void*)0);
		glEnableVertexAttribArray(0);
		glEnableVertexAttribArray(1);

		va.Unbind();
		vertexBuffer.Unbind(false);
		elementBuffer.Unbind(true);

		renderComponent.VAO = va.getID();
		renderComponent.IBO = elementBuffer.getBufferID();
	});
}



void Renderer::Render(Scene& scene)
{
	auto group = scene.getReg().view<TransformComponent, Render3DComponent>();
	group.each([&](auto entity , TransformComponent& trans, Render3DComponent& renderComponent) 
		{
		glBindVertexArray(renderComponent.VAO);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, renderComponent.IBO);

		renderComponent.shader.Use();

		auto lights = scene.getReg().view<LightComponent>();
		lights.each([&](auto entity, LightComponent& light) {
		  	renderComponent.shader.setUniformVec3("lighting.color", light.Color);
			renderComponent.shader.setUniformFloat("lighting.strength", light.Strength);
		});


		glm::mat4 rotation = glm::rotate(glm::mat4(1.0f), trans.Rotation.x, glm::vec3(1.0f, 0.0f, 0.0f));
		rotation *= glm::rotate(glm::mat4(1.0f), trans.Rotation.y, glm::vec3(0.0f, 1.0f, 0.0f));
		rotation *= glm::rotate(glm::mat4(1.0f), trans.Rotation.z, glm::vec3(0.0f, 0.0f, 1.0f));


		glm::mat4 modelMatrix = glm::translate(glm::mat4(1.0f), trans.Position) * glm::scale(glm::mat4(1.0f), trans.Scale) *rotation;


		renderComponent.shader.setUniformVec3("Color", renderComponent.color);
		renderComponent.shader.setUniformMat4("M", modelMatrix);
		renderComponent.shader.setUniformMat4("V", cam.GetViewMatrix());
		renderComponent.shader.setUniformMat4("P", projection);

		//std::cout << "Draw " << renderComponent.mesh.elements.size() << " elements" << std::endl;
		glDrawElements(GL_TRIANGLES, static_cast<unsigned int>(renderComponent.mesh.elements.size()) ,
			GL_UNSIGNED_INT, NULL);

		glBindVertexArray(0);
	});

}

void Renderer::Render(Framebuffer& framebuffer, Scene& scene)
{

	glBindFramebuffer(GL_FRAMEBUFFER, framebuffer.GetId());

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	Render(scene);
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
}



}